<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    {% if config.site_description %}<meta name="description" content="{{ config.site_description }}">{% endif %}
    {% if config.site_author %}<meta name="author" content="{{ config.site_author }}">{% endif %}
    {% if page.canonical_url %}<link rel="canonical" href="{{ page.canonical_url }}">{% endif %}
    <link rel="shortcut icon" href="{{ 'img/favicon.ico'|url }}">

    {% block htmltitle %}
    <title>{% if page.title %}{{ page.title }} - {% endif %}{{ config.site_name }}</title>
    {% endblock %}

    <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.12.0/css/all.css">
    <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.12.0/css/v4-shims.css">
    <link rel="stylesheet" href="//cdn.jsdelivr.net/npm/hack-font@3.3.0/build/web/hack.min.css">
    <link href='//rsms.me/inter/inter.css' rel='stylesheet' type='text/css'>
    <link href='//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,700italic,400,300,600,700&subset=latin-ext,latin' rel='stylesheet' type='text/css'>
    <link href="{{ 'css/bootstrap-custom.min.css'|url }}" rel="stylesheet">
    <link href="{{ 'css/base.min.css'|url }}" rel="stylesheet">
    <link href="{{ 'css/cinder.min.css'|url }}" rel="stylesheet">

    {% if config.theme.highlightjs is defined and config.theme.highlightjs is sameas false %} 

    {% else %}
        {% if config.theme.colorscheme %}
        <link rel="stylesheet" href="//cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.18.0/build/styles/{{ config.theme.colorscheme }}.min.css">
        {% else %}
        <link rel="stylesheet" href="//cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.18.0/build/styles/github.min.css">
        {% endif %}
    {% endif %}

    {%- for path in config['extra_css'] %}
    <link href="{{ path|url }}" rel="stylesheet">
    {%- endfor %}

    <!-- HTML5 shim and Respond.js IE8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
            <script src="https://cdn.jsdelivr.net/npm/html5shiv@3.7.3/dist/html5shiv.min.js"></script>
            <script src="https://cdn.jsdelivr.net/npm/respond.js@1.4.2/dest/respond.min.js"></script>
        <![endif]-->

    {% if config.google_analytics %}
    <script>
    (function(i, s, o, g, r, a, m) {
        i['GoogleAnalyticsObject'] = r;
        i[r] = i[r] || function() {
            (i[r].q = i[r].q || []).push(arguments)
        }, i[r].l = 1 * new Date();
        a = s.createElement(o),
        m = s.getElementsByTagName(o)[0];
        a.async = 1;
        a.src = g;
        m.parentNode.insertBefore(a, m)
    })(window, document, 'script', '//www.google-analytics.com/analytics.js', 'ga');

    ga('create', '{{ config.google_analytics[0] }}', '{{ config.google_analytics[1] }}');
    ga('send', 'pageview');
    </script>
    {% endif %}

    {% block extrahead %} {% endblock %}
</head>

<body{% if page and page.is_homepage %} class="homepage" {% endif %}>

    {% include "nav.html" %}

    <div class="container">
        {% block content %}
        {% if page.meta.disable_toc %}
        <div class="col-md-12" role="main">{% include "content.html" %}</div>
        {% else %}
        <div class="col-md-3">{% include "toc.html" %}</div>
        <div class="col-md-9" role="main">{% include "content.html" %}</div>
        {% endif %}
        {% endblock %}
    </div>

    {% if not config.theme.disable_footer %}
      <footer class="col-md-12 text-center">
          {% block footer %}
          {% if not config.theme.disable_footer_except_revision %}
            <hr>
            <p>{% if config.copyright %}
            <small>{{ config.copyright }}</small><br>
            {% endif %}
            <small>Documentation built with <a href="http://www.mkdocs.org/">MkDocs</a>.</small>
            </p>
          {% endif %}

          {% if page and page.meta.revision_date %}
          {% if config.theme.disable_footer_except_revision %}<hr>{% else %}<br>{% endif %}
            <small>Revised on: {{ page.meta.revision_date }}</small>
          {% endif %}
          {% endblock %}
      </footer>
    {% endif %}

    {%- block scripts %}
    <script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>
    <script src="{{ 'js/bootstrap-3.0.3.min.js'|url }}"></script>

    {% if config.theme.highlightjs is defined and config.theme.highlightjs is sameas false %}  <!-- default is to include highlightjs -->
    
    {% else %}
    <script src="//cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.18.0/build/highlight.min.js"></script>
        {% if config.theme.hljs_languages %}
            {%- for lang in config.theme.hljs_languages %}
                <script src="//cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.18.0/build/languages/{{lang}}.min.js"></script>
            {%- endfor %}
        {% endif %}
    <script>hljs.initHighlightingOnLoad();</script>
    {% endif %}

    <script>var base_url = {{ base_url | tojson }}</script>
    {% if config.shortcuts %}
        <script>var shortcuts = {{ config.shortcuts | tojson }}</script>
    {% endif %}
    <script src="{{ 'js/base.js'|url }}"></script>
    {%- for path in config['extra_javascript'] %}
    <script src="{{ path|url }}"></script>
    {%- endfor %}
    {%- endblock %}

    {% if 'search' in config['plugins'] %}{%- include "search-modal.html" %}{% endif %}
    {%- include "keyboard-modal.html" %}
    </body>

</html>
{% if page and page.is_homepage %}
<!--
MkDocs version : {{ mkdocs_version }}
Build Date UTC : {{ build_date_utc }}
-->
{% endif %}
