/*
  Cinder Theme for MkDocs | Copyright 2015 Christopher Simpkins | MIT License
*/

body {
    font-family:"Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-size: 16px;
    line-height: 1.7;
    background-color: #FFF;
    color: #343838;
}
h1, h2, h3, h4, h5, h6 {
    font-family:'Inter', 'Helvetica Neue', Helvetica, Arial, sans-serif;
    color: #222;
}
h1 small, h2 small, h3 small, h4 small, h5 small, h6 small, .h1 small, .h2 small, .h3 small, .h4 small, .h5 small, .h6 small, h1 .small, h2 .small, h3 .small, h4 .small, h5 .small, h6 .small, .h1 .small, .h2 .small, .h3 .small, .h4 .small, .h5 .small, .h6 .small {
    color: #B1B7B9;
}

h2 {
    margin-top: 35px;
}

h1, h2 {
    font-weight: 700;
}
h4 {
    font-family: 'Inter', 'Helvetica Neue', Helvetica, Arial, sans-serif;
    font-weight: 300;
    margin-top: 20px;
    font-style: italic;
}
h5 {
    font-family: 'Inter', 'Helvetica Neue', Helvetica, Arial, sans-serif;
    font-weight: 300;
    font-variant: small-caps;
}
pre, code {
    background-color: #FCFDFF;
}
pre>code {
    font-size: 13px;
}
pre {
    margin-top: 25px;
    margin-bottom: 25px;
}
.lead {
    font-family:"Inter", "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-weight: 400;
    line-height: 1.4;
    letter-spacing: 0.0312em;
    color: #B1B7B9;
}
.navbar-default {
    background-color: #343838;
    border-bottom: 8px #EBF2F2 solid;
}
.bs-sidenav {
    background-image: url("../img/grid11.png");
    background-repeat: repeat;
    font-family: Inter,"Helvetica Neue",Helvetica,Arial,sans-serif;
    font-size: 13px;
}
.well {
    background-color: #FCFDFF;
}
.btn-default {
    background-color:#FCFDFF;
}
.table-striped > tbody > tr:nth-child(2n+1) > td, .table-striped > tbody > tr:nth-child(2n+1) > th {
    background-color: #FCFDFF;
}
#mkdocs-search-query:focus {
    outline: none;
    -webkit-box-shadow: none;
    box-shadow: none;
}
#mkdocs-search-query {
    font-family:"Inter", "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-size: 20px;
    font-weight: 700;
    color: #343838;
    height: 45px;
}
footer > hr {
    width: 35%;
}
